﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Data.SqlClient;
using System.Linq;

using MvcCms.Data;


namespace MvcCmsProjectTracker.Data
{
    public class EFProjectRepository : IProjectRepository
    {
        #region Constructors
        private readonly MvcCmsEntities _entities;

        public EFProjectRepository(IEntitiesProvider entitiesProvider)
        {
            _entities = entitiesProvider.GetEntityContext();
        }
        #endregion

        public void Save()
        {
            _entities.SaveChanges();
        }

        public IQueryable<Project> GetAllProjects(int portalId)
        {
            return from p in _entities.Projects
                   where p.Deleted == false && p.PortalID==portalId
                   select p;
        }
        public IQueryable<HelpPage> GetHelpPages(int portalId)
        {
            return from p in _entities.HelpPages
                   .Include("ContentPage.ContentCategory")
                   where p.PortalID == portalId
                   select p;
        }
        public IQueryable<Project> GetMyProjects(Guid userId)
        {
            return from p in _entities.Projects
                   join m in _entities.ProjectMembers on p.ProjectID equals m.ProjectID
                   where m.UserID==userId
                   && p.Deleted == false
                   && !p.IsForHelpDesk
                   select p;
        }

        public IQueryable<Project> GetHelpDeskProjects(int portalId)
        {
            return from p in _entities.Projects
                   where p.IsForHelpDesk
                   && p.Deleted == false
                   && p.PortalID==portalId
                   select p;
        }

        public IQueryable<Project> GetNonHelpDeskProjects(int portalId)
        {
            return from p in _entities.Projects
                   where !p.IsForHelpDesk
                   && p.Deleted == false
                   select p;
        }

        public Project GetProject(int projectID)
        {
            return _entities.Projects.SingleOrDefault(p => p.ProjectID == projectID);
        }

        public void AddProject(Project project)
        {
            _entities.Projects.AddObject(project);
        }
        public void AddHelpPage(HelpPage helpPage)
        {
            _entities.HelpPages.AddObject(helpPage);
        }
        public void DeleteProject(Project project)
        {
            project.Deleted = true;
        }

        public IQueryable<ProjectMilestone> GetProjectMilestones(int projectID)
        {
            return from m in _entities.ProjectMilestones
                   where m.ProjectID == projectID
                   select m;
        }

        public ProjectMilestone GetProjectMilestone(int milestoneID)
        {
            return _entities.ProjectMilestones.SingleOrDefault(m => m.MilestoneID == milestoneID);
        }

        public void AddProjectMilestone(ProjectMilestone milestone)
        {
            _entities.ProjectMilestones.AddObject(milestone);
        }

        public void DeleteProjectMilestone(ProjectMilestone milestone)
        {
            var bugs = from b in _entities.Bugs
                       where b.ProjectID == milestone.ProjectID
                       && b.MilestoneID == milestone.MilestoneID
                       select b;
            foreach (Bug b in bugs)
            {
                b.MilestoneID = null;
            }
            _entities.ProjectMilestones.DeleteObject(milestone);
        }

        public IQueryable<ProjectCategory> GetProjectCategories(int projectID)
        {
            return from c in _entities.ProjectCategories
                   where c.ProjectID == projectID
                   select c;
        }

        public ProjectCategory GetProjectCategory(int categoryID)
        {
            return _entities.ProjectCategories.SingleOrDefault(c => c.CategoryID == categoryID);
        }

        public void AddProjectCategory(ProjectCategory category)
        {
            _entities.ProjectCategories.AddObject(category);
        }

        public void DeleteProjectCategory(ProjectCategory category)
        {
            var bugs = from b in _entities.Bugs
                       where b.ProjectID == category.ProjectID
                       && b.CategoryID == category.CategoryID
                       select b;
            foreach (Bug b in bugs)
            {
                b.CategoryID = null;
            }
            _entities.ProjectCategories.DeleteObject(category);
        }

        public IQueryable<ProjectPriority> GetProjectPriorities(int projectID)
        {
            return from p in _entities.ProjectPriorities
                   where p.ProjectID == projectID
                   select p;
        }

        public ProjectPriority GetProjectPriority(int priorityID)
        {
            return _entities.ProjectPriorities.SingleOrDefault(p => p.PriorityID == priorityID);
        }

        public void AddProjectPriority(ProjectPriority priority)
        {
            _entities.ProjectPriorities.AddObject(priority);
        }

        public void DeleteProjectPriority(ProjectPriority priority)
        {
            var bugs = from b in _entities.Bugs
                       where b.ProjectID == priority.ProjectID
                       && b.PriorityID == priority.PriorityID
                       select b;
            foreach (Bug b in bugs)
            {
                b.PriorityID = null;
            }
            _entities.ProjectPriorities.DeleteObject(priority);
        }

        public IQueryable<ProjectStatus> GetProjectStatuses(int projectID)
        {
            return from s in _entities.ProjectStatuses
                   where s.ProjectID == projectID
                   select s;
        }

        public ProjectStatus GetProjectStatus(int statusID)
        {
            return _entities.ProjectStatuses.SingleOrDefault(s => s.StatusID == statusID);
        }

        public void AddProjectStatus(ProjectStatus status)
        {
            _entities.ProjectStatuses.AddObject(status);
        }

        public void DeleteProjectStatus(ProjectStatus status)
        {
            var bugs = from b in _entities.Bugs
                       where b.ProjectID == status.ProjectID
                       && b.StatusID == status.StatusID
                       select b;
            foreach (Bug b in bugs)
            {
                b.StatusID = null;
            }
            _entities.ProjectStatuses.DeleteObject(status);
        }

        public IQueryable<ProjectCustomField> GetProjectCustomFields(int projectID)
        {
            return from cf in _entities.ProjectCustomFields
                   where cf.ProjectID == projectID
                   && cf.Deleted == false
                   select cf;
        }

        public ProjectCustomField GetProjectCustomField(int customFieldID)
        {
            return _entities.ProjectCustomFields.SingleOrDefault(cf => cf.CustomFieldID == customFieldID);
        }

        public void AddProjectCustomField(ProjectCustomField customField)
        {
            _entities.ProjectCustomFields.AddObject(customField);
        }

        public void DeleteProjectCustomField(ProjectCustomField customField)
        {
            customField.Deleted = true;
        }

        public IQueryable<ProjectCustomFieldValue> GetProjectCustomFieldValues(int projectID)
        {
            throw new NotImplementedException();
        }

        public ProjectCustomFieldValue GetProjectCustomFieldValue(int customFieldID)
        {
            throw new NotImplementedException();
        }

        public void AddProjectCustomFieldValue(ProjectCustomFieldValue customFieldValue)
        {
            throw new NotImplementedException();
        }

        public void DeleteProjectCustomFieldValue(ProjectCustomFieldValue customFieldValue)
        {
            throw new NotImplementedException();
        }

        public IQueryable<ProjectMember> GetProjectMembers(int projectID)
        {
            return from m in _entities.ProjectMembers
                   .Include("User")
                   where m.ProjectID == projectID
                   orderby m.User.UserName ascending
                   select m;
        }

        public ProjectMember GetProjectMember(int projectID, Guid userId)
        {
            return (from m in _entities.ProjectMembers
                    .Include("User")
                    where m.ProjectID == projectID
                    && m.UserID==userId
                    select m).SingleOrDefault();
        }

        public ProjectMember GetProjectMember(Guid userId)
        {
            return (from m in _entities.ProjectMembers
                    where m.UserID == userId
                    select m).SingleOrDefault();
        }

        public void AddProjectMember(ProjectMember projectMember)
        {
            _entities.ProjectMembers.AddObject(projectMember);
        }

        public void DeleteProjectMember(ProjectMember projectMember)
        {
            _entities.ProjectMembers.DeleteObject(projectMember);
        }
    }
}