﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.Specialized;
using System.Configuration;
using System.Web.Configuration;
using System.Web.Hosting;

namespace MvcCms.Service.Code.Membership
{
    public static class CacheConfig
    {

        private static string GetConfigValue(string configValue, string defaultValue)
        {
            return String.IsNullOrEmpty(configValue) ? defaultValue : configValue;
        }
        public static NameValueCollection SetMemberShipConfig()
        {
            var webConfig = WebConfigurationManager.OpenWebConfiguration(null);
            var configSection = (MembershipSection)webConfig.GetSection("system.web/membership");
            NameValueCollection config = (NameValueCollection)configSection.Providers[0].Parameters;  

            if (String.IsNullOrEmpty(config["description"]))
            {
                config.Remove("description");
                config.Add("description", "MvcCms Membership Provider");
            }

            config["applicationName"] = GetConfigValue(config["applicationName"], HostingEnvironment.ApplicationVirtualPath);
            config["maxInvalidPasswordAttempts"] = GetConfigValue(config["maxInvalidPasswordAttempts"], "5");
            config["passwordAttemptWindow"] = GetConfigValue(config["passwordAttemptWindow"], "10");
            config["minRequiredNonAlphanumericCharacters"] = GetConfigValue(config["minRequiredNonAlphanumericCharacters"], "1");
            config["minRequiredPasswordLength"] = GetConfigValue(config["minRequiredPasswordLength"], "7");
            config["passwordStrengthRegularExpression"] = GetConfigValue(config["passwordStrengthRegularExpression"], "");
            config["enablePasswordReset"] = GetConfigValue(config["enablePasswordReset"], "true");
            config["enablePasswordRetrieval"] = GetConfigValue(config["enablePasswordRetrieval"], "true");
            config["requiresQuestionAndAnswer"] = GetConfigValue(config["requiresQuestionAndAnswer"], "false");
            config["requiresUniqueEmail"] = GetConfigValue(config["requiresUniqueEmail"], "true");

            config["passwordFormat"] = config["passwordFormat"] ?? "Hashed";

            // Get encryption and decryption key information from the configuration.
            Configuration cfg = WebConfigurationManager.OpenWebConfiguration(HostingEnvironment.ApplicationVirtualPath);
            var machineKey = (MachineKeySection)cfg.GetSection("system.web/machineKey");
            config["validationKey"] = machineKey.ValidationKey;
            return config;
        }
    }
}
