﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;

using MvcCmsForum.Data;

namespace MvcCms.Areas.MvcCmsForum.Views.Shared.ControlsForum
{
    public partial class Pager : ViewUserControl<DTOAjaxPost>
	{
		protected const int PageBuffer = 3;
		private string _currentAction = String.Empty;

		/// <summary>
		/// Gets the current action.
		/// </summary>
		/// <value>The current action.</value>
		public string CurrentAction
		{
			get
			{
				if (String.IsNullOrEmpty(_currentAction))
					_currentAction = Html.ViewContext.RouteData.Values["action"] as string;

				return _currentAction;
			}
		}

		/// <summary>
		/// Gets the page URL.
		/// </summary>
		/// <param name="page">The page.</param>
		/// <returns></returns>
		public string GetPageUrl(int page)
		{
			if (page < 1)
				throw new ArgumentOutOfRangeException("page", "'page' must be greater than or equal to 1");
            

            var forumpath = this.ViewContext.RouteData.Values["forumpath"].ToString();
            var path = this.ViewContext.RouteData.Values["path"].ToString();

            string url = "/forums/forum-" + forumpath + "/" + path;
            url += "/page" + page.ToString();

            return url;			
		}

		/// <summary>
		/// Gets the start and end page.
		/// </summary>
		/// <param name="start">The start.</param>
		/// <param name="end">The end.</param>
		public void GetStartAndEndPage(out int start, out int end)
		{
			if (ViewData.Model.TotalPages <= PageBuffer)
			{
				start = 1;
                end = ViewData.Model.TotalPages;
			}
			else
			{
				start = Math.Max(ViewData.Model.PageNumber - PageBuffer, 1);
				end = Math.Min(ViewData.Model.PageNumber + PageBuffer, ViewData.Model.TotalPages);
			}
		}
	}
}
